#include "customcanvas.h"
#include "QVBoxLayout"

CustomCanvas::CustomCanvas(QWidget *parent)
    : ICanvas(parent)
    , m_data(0)
{

    m_customPlot = new QCustomPlot(this);
    m_customPlot->setObjectName(QString::fromUtf8("customPlot"));

    QVBoxLayout *mainLayout = new QVBoxLayout;
    //mainLayout->addWidget(canvas);
    mainLayout->addWidget(m_customPlot);
    setLayout(mainLayout);

}


void CustomCanvas::Draw(const OutputData<double> *data)
{
    Q_ASSERT(data);

    if(data->getRank() != 2) {
        throw NullPointerException("CustomCanvas::Draw() -> Error. Zero pointer to the data to draw");
    }

    if(data == m_data) return;
    m_data = data;

    m_customPlot->setInteractions(QCP::iRangeDrag|QCP::iRangeZoom); // this will also allow rescaling the color scale by dragging/zooming
    m_customPlot->axisRect()->setupFullAxesBox(true);
    m_customPlot->xAxis->setLabel("x");
    m_customPlot->yAxis->setLabel("y");

    const IAxis *axis_x = data->getAxis(0);
    const IAxis *axis_y = data->getAxis(1);

    // set up the QCPColorMap:
    QCPColorMap *colorMap = new QCPColorMap(m_customPlot->xAxis, m_customPlot->yAxis);
    m_customPlot->addPlottable(colorMap);

    int nx = axis_x->getSize();
    int ny = axis_y->getSize();
    colorMap->data()->setSize(nx, ny); // we want the color map to have nx * ny data points
    colorMap->data()->setRange(QCPRange(axis_x->getMin(), axis_x->getMax()), QCPRange(axis_y->getMin(), axis_y->getMax())); // and span the coordinate range -4..4 in both key (x) and value (y) dimensions


    OutputData<double>::const_iterator it = data->begin();
    while (it != data->end()) {
        std::vector<int> indices =
                data->toCoordinates(it.getIndex());

        colorMap->data()->setCell(indices[0], indices[1], *it);
        ++it;
    }


    // add a color scale:
    QCPColorScale *colorScale = new QCPColorScale(m_customPlot);
    m_customPlot->plotLayout()->addElement(0, 1, colorScale); // add it to the right of the main axis rect

    colorScale->setDataScaleType(QCPAxis::stLogarithmic);

    colorScale->setType(QCPAxis::atRight); // scale shall be vertical bar with tick/axis labels right (actually atRight is already the default)
    colorMap->setColorScale(colorScale); // associate the color map with the color scale
    colorScale->axis()->setLabel("Magnetic Field Strength");

    // set the color gradient of the color map to one of the presets:
    colorMap->setGradient(QCPColorGradient::gpPolar);
    // we could have also created a QCPColorGradient instance and added own colors to
    // the gradient, see the documentation of QCPColorGradient for what's possible.

    // rescale the data dimension (color) such that all data points lie in the span visualized by the color gradient:
    colorMap->rescaleDataRange();

    // make sure the axis rect and color scale synchronize their bottom and top margins (so they line up):
    QCPMarginGroup *marginGroup = new QCPMarginGroup(m_customPlot);
    m_customPlot->axisRect()->setMarginGroup(QCP::msBottom|QCP::msTop, marginGroup);
    colorScale->setMarginGroup(QCP::msBottom|QCP::msTop, marginGroup);

    // rescale the key (x) and value (y) axes so the whole color map is visible:
    m_customPlot->rescaleAxes();
}
